@@ -220,7 +220,8 @@ class InstallRequirement(object):
         finally:
             logger.indent -= 2
         if not self.req:
-            self.req = pkg_resources.Requirement.parse(self.pkg_info()['Name'])
+            self.req = pkg_resources.Requirement.parse(
+                "%(Name)s==%(Version)s" % self.pkg_info())
             self.correct_build_location()
 
     ## FIXME: This is a lame hack, entirely for PasteScript which has
@@ -728,6 +729,37 @@ deleted (unless you remove this file).
 '''
 
 
+class Requirements(object):
+
+    def __init__(self):
+        self._keys = []
+        self._dict = {}
+
+    def keys(self):
+        return self._keys
+
+    def values(self):
+        values_list = []
+        for key in self._keys:
+            values_list.append(self._dict[key])
+        return values_list
+
+    def __contains__(self, item):
+        return item in self._keys
+    
+    def __setitem__(self, key, value):
+        if key not in self._keys:
+            self._keys.append(key)
+        self._dict[key] = value
+    
+    def __getitem__(self, key):
+        return self._dict[key]
+
+    def __repr__(self):
+        values = [ '%s: %s' % (repr(k), repr(self[k])) for k in self.keys() ]
+        return 'Requirements({%s})' % ', '.join(values)
+
+
 class RequirementSet(object):
 
     def __init__(self, build_dir, src_dir, download_dir, download_cache=None,
@@ -739,7 +771,7 @@ class RequirementSet(object):
         self.download_cache = download_cache
         self.upgrade = upgrade
         self.ignore_installed = ignore_installed
-        self.requirements = {}
+        self.requirements = Requirements()
         # Mapping of alias: real_name
         self.requirement_aliases = {}
         self.unnamed_requirements = []
@@ -947,6 +979,16 @@ class RequirementSet(object):
                             #@@ sketchy way of identifying packages not grabbed from an index
                             if bundle and req_to_install.url:
                                 self.copy_to_build_dir(req_to_install)
+                                install = False
+                        # req_to_install.req is only avail after unpack for URL pkgs
+                        # repeat check_if_exists to uninstall-on-upgrade (#14)
+                        req_to_install.check_if_exists()
+                        if req_to_install.satisfied_by:
+                            if self.upgrade:
+                                req_to_install.conflicts_with = req_to_install.satisfied_by
+                                req_to_install.satisfied_by = None
+                            else:
+                                install = False
                 if not is_bundle and not self.is_download:
                     ## FIXME: shouldn't be globally added:
                     finder.add_dependency_links(req_to_install.dependency_links)
@@ -1023,9 +1065,9 @@ class RequirementSet(object):
 
     def install(self, install_options, global_options=()):
         """Install everything in this set (after having downloaded and unpacked the packages)"""
-        to_install = sorted([r for r in self.requirements.values()
-                             if self.upgrade or not r.satisfied_by],
-                            key=lambda p: p.name.lower())
+        to_install = [r for r in self.requirements.values()
+                      if self.upgrade or not r.satisfied_by]
+
         if to_install:
             logger.notify('Installing collected packages: %s' % (', '.join([req.name for req in to_install])))
         logger.indent += 2
@@ -1113,16 +1155,12 @@ class RequirementSet(object):
 
     def bundle_requirements(self):
         parts = [self.BUNDLE_HEADER]
-        for req in sorted(
-            [req for req in self.requirements.values()
-             if not req.comes_from],
-            key=lambda x: x.name):
+        for req in [req for req in self.requirements.values()
+                    if not req.comes_from]:
             parts.append('%s==%s\n' % (req.name, req.installed_version))
         parts.append('# These packages were installed to satisfy the above requirements:\n')
-        for req in sorted(
-            [req for req in self.requirements.values()
-             if req.comes_from],
-            key=lambda x: x.name):
+        for req in [req for req in self.requirements.values()
+                    if req.comes_from]:
             parts.append('%s==%s\n' % (req.name, req.installed_version))
         ## FIXME: should we do something with self.unnamed_requirements?
         return ''.join(parts)
@@ -1248,7 +1286,7 @@ def parse_editable(editable_req, default_vcs=None):
     else:
         req = match.group(1)
     ## FIXME: use package_to_requirement?
-    match = re.search(r'^(.*?)(?:-dev|-\d.*)', req)
+    match = re.search(r'^(.*?)(?:-dev|-\d.*)$', req)
     if match:
         # Strip off -dev, -0.2, etc.
         req = match.group(1)
@@ -1366,7 +1404,7 @@ class UninstallPathSet(object):
     def commit(self):
         """Remove temporary save dir: rollback will no longer be possible."""
         if self.save_dir is not None:
-            shutil.rmtree(self.save_dir)
+            rmtree(self.save_dir)
             self.save_dir = None
             self._moved_paths = []
 
